@@ -1,37 +1,30 @@
 #!/usr/bin/env python3
 
 """
-This script simply wraps terragrunt (which is a wrapper around terraform...) and its main function is
-to allow you to execute a `run-all` command but broken up in individual steps.
+This script simply wraps terragrunt (which is a wrapper around terraform...)
+and its main function is to allow you to execute a `run-all` command but
+broken up in individual steps.
 
-This makes debugging a complex project easier, such as spotting where the exact problem is.
+This makes debugging a complex project easier, such as spotting where the
+exact problem is.
 """
 
-# Todo: parse output
-# - https://github.com/bcochofel/terraplanfeed/tree/main/terraplanfeed
-
-import os
 import sys
-import subprocess
-import shlex
-import json
-import re
+from importlib.metadata import version, PackageNotFoundError
+
 import click
-import networkx as nx
 
 from outdated import check_outdated # type: ignore
 
 from .main import TgWrap
 
-from importlib.metadata import version, PackageNotFoundError
-__name__ = "tgwrap"
 try:
     __version__ = version(__name__)
 except PackageNotFoundError:
     __version__ = '0.0.0'
 
 def check_latest_version():
-    # check for newer versions
+    """ check for later versions on pypi """
     try:
         is_outdated, latest_version = check_outdated(__name__, __version__)
         if is_outdated:
@@ -42,9 +35,17 @@ def check_latest_version():
         # this happens when your local version is ahead of the pypi version,
         # which happens only in development
         pass
+    except KeyError:
+        click.secho('Could not determine package version, continue nevertheless.', bold=True, file=sys.stderr)
+        pass
 
 CLICK_CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
 
+TG_COMMANDS=[
+    'init', 'validate', 'validate-inputs', 'plan', 'apply',
+    'destroy', 'info', 'output', 'show'
+    ]
+
 class DefaultGroup(click.Group):
     '''
     Allow a default command for a group
@@ -53,12 +54,13 @@ class DefaultGroup(click.Group):
 
     def __init__(self, *args, **kwargs):
         default_command = kwargs.pop('default_command', None)
-        super(DefaultGroup, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.default_cmd_name = None
         if default_command is not None:
             self.set_default_command(default_command)
 
     def set_default_command(self, command):
+        """ Sets the command that can be omitted (and is considered default) """
         if isinstance(command, str):
             cmd_name = command
         else:
@@ -69,16 +71,16 @@ class DefaultGroup(click.Group):
     def parse_args(self, ctx, args):
         if not args and self.default_cmd_name is not None:
             args.insert(0, self.default_cmd_name)
-        return super(DefaultGroup, self).parse_args(ctx, args)
+        return super().parse_args(ctx, args)
 
     def get_command(self, ctx, cmd_name):
         if cmd_name not in self.commands and self.default_cmd_name is not None:
             ctx.args0 = cmd_name
             cmd_name = self.default_cmd_name
-        return super(DefaultGroup, self).get_command(ctx, cmd_name)
+        return super().get_command(ctx, cmd_name)
 
     def resolve_command(self, ctx, args):
-        cmd_name, cmd, args = super(DefaultGroup, self).resolve_command(ctx, args)
+        cmd_name, cmd, args = super().resolve_command(ctx, args)
         args0 = getattr(ctx, 'args0', None)
         if args0 is not None:
             args.insert(0, args0)
@@ -98,9 +100,7 @@ def main():
         ignore_unknown_options=True,
     ),
 )
-@click.argument('command', type=click.Choice(
-    ['init', 'validate', 'validate-inputs', 'plan', 'apply', 'destroy', 'info', 'output', 'state', 'show']
-    ))
+@click.argument('command', type=click.Choice(TG_COMMANDS + ['state', 'render-json']))
 @click.option('--verbose', '-v', is_flag=True, default=False,
     help='Verbose printing',
     show_default=True
@@ -110,7 +110,7 @@ def main():
     show_default=True
     )
 @click.option('--dry-run', '-D', is_flag=True, default=False,
-    help='Run in a dry-run mode, no real actions are executed. Applies only in combination with step-by-step mode.',
+    help='dry-run mode, no real actions are executed (only in combination with step-by-step mode)',
     show_default=True
     )
 @click.option('--no-lock', '-n', is_flag=True, default=False,
@@ -131,7 +131,8 @@ def main():
     )
 @click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
 @click.version_option(version=__version__)
-def run(command, verbose, debug, dry_run, no_lock, update_source, auto_approve, working_dir, terragrunt_args):
+def run(command, verbose, debug, dry_run, no_lock, update_source,
+    auto_approve, working_dir, terragrunt_args):
     """ [default] Executes a terragrunt command on a single project """
 
     check_latest_version()
@@ -154,9 +155,7 @@ def run(command, verbose, debug, dry_run, no_lock, update_source, auto_approve,
         ignore_unknown_options=True,
     ),
 )
-@click.argument('command', type=click.Choice(
-    ['init', 'validate', 'plan', 'apply', 'destroy', 'info', 'output', 'show']
-    ))
+@click.argument('command', type=click.Choice(TG_COMMANDS))
 @click.option('--verbose', '-v', is_flag=True, default=False,
     help='Verbose printing',
     show_default=True
@@ -166,7 +165,7 @@ def run(command, verbose, debug, dry_run, no_lock, update_source, auto_approve,
     show_default=True
     )
 @click.option('--dry-run', '-D', is_flag=True, default=False,
-    help='Run in a dry-run mode, no real actions are executed. Applies only in combination with step-by-step mode.',
+    help='dry-run mode, no real actions are executed (only in combination with step-by-step mode)',
     show_default=True
     )
 @click.option('--no-lock', '-n', is_flag=True, default=False,
@@ -221,7 +220,7 @@ def run_all(command, verbose, debug, dry_run, no_lock, update_source, exclude_ex
     show_default=True
     )
 @click.option('--dry-run', '-D', is_flag=True, default=False,
-    help='Run in a dry-run mode, no real actions are executed. Applies only in combination with step-by-step mode.',
+    help='dry-run mode, no real actions are executed (only in combination with step-by-step mode)',
     show_default=True
     )
 @click.option('--exclude-external-dependencies', '-x', is_flag=True, default=False,
