@@ -1,13 +1,15 @@
 #!/usr/bin/env python3
 
 """
-This script simply wraps terragrunt (which is a wrapper around terraform...) and its main function is
-to allow you to execute a `run-all` command but broken up in individual steps.
+This script simply wraps terragrunt (which is a wrapper around terraform...)
+and its main function is to allow you to execute a `run-all` command but
+broken up in individual steps.
 
-This makes debugging a complex project easier, such as spotting where the exact problem is.
+This makes debugging a complex project easier, such as spotting where the
+exact problem is.
 """
 
-# Todo: parse output
+# idea: parse output
 # - https://github.com/bcochofel/terraplanfeed/tree/main/terraplanfeed
 
 import os
@@ -19,8 +21,6 @@ import re
 import click
 import networkx as nx
 
-from importlib.metadata import version, PackageNotFoundError
-
 class Printer():
     """ A simple class for printing nice messages """
     def __init__(
@@ -28,38 +28,42 @@ class Printer():
         verbose: bool,
         ):
 
-        self.verbose = verbose
+        self._print_verbose = verbose
 
-    def print_header(self, msg):
+    def header(self, msg):
         msg = msg.strip() if isinstance(msg, str) else msg
         click.secho('\n' + msg, bold=True, file=sys.stderr)
 
-    def print_verbose(self, msg):
-        if self.verbose:
+    def verbose(self, msg):
+        if self._print_verbose:
             msg = msg.strip() if isinstance(msg, str) else msg
             print(msg, flush=True, file=sys.stderr)
 
-    def print_normal(self, msg):
+    def normal(self, msg):
         msg = msg.strip() if isinstance(msg, str) else msg
         print(msg, flush=True, file=sys.stderr)
 
-    def print_bold(self, msg):
+    def bold(self, msg):
         msg = msg.strip() if isinstance(msg, str) else msg
         click.secho('\n' + msg, bold=True, file=sys.stderr)
 
-    def print_warning(self, msg):
+    def warning(self, msg):
         msg = msg.strip() if isinstance(msg, str) else msg
         click.secho('\n' + msg + '\n', fg="yellow", bold=True, file=sys.stderr)
 
-    def print_error(self, msg):
+    def error(self, msg):
         msg = msg.strip() if isinstance(msg, str) else msg
         click.secho(msg, fg="red", bold=True, file=sys.stderr)
 
-    def print_success(self, msg):
+    def success(self, msg):
         msg = msg.strip() if isinstance(msg, str) else msg
         click.secho(msg, fg="green", bold=True, file=sys.stderr)
 
 class TgWrap():
+    """
+    A wrapper around terragrunt with the sole purpose to make it a bit
+    (in an opiionated way) easier to use
+    """
     SEPARATOR=':|:'
 
     def __init__(self, verbose):
@@ -68,9 +72,13 @@ class TgWrap():
         # Check if the "TERRAGRUNT_SOURCE" environment variable is set
         env_var = "TERRAGRUNT_SOURCE"
         if env_var in os.environ:
-            self.printer.print_warning(f"'{env_var}' environment variable is set with address: '{os.environ[env_var]}'!")
+            self.printer.warning(
+                f"'{env_var}' environment variable is set with address: '{os.environ[env_var]}'!"
+                )
         else:
-            self.printer.print_warning(f"No '{env_var}' variable is set, so the sources as defined in terragrunt.hcl files will be used as is!")
+            self.printer.warning(
+                f"No '{env_var}' variable is set, so the sources as defined in terragrunt.hcl files will be used as is!"
+                )
 
     def prepare_groups(self, graph, exclude_external_dependencies):
         """ Prepare the list of groups that will be executed """
@@ -78,16 +86,18 @@ class TgWrap():
         for group in nx.topological_generations(graph):
             try:
                 group.remove("\\n") # terragrunt is adding this in some groups for whatever reason
-            except ValueError as e:
+            except ValueError:
                 pass
-            for idx, dir in enumerate(group):
-                if dir.startswith("/") \
+            for idx, directory in enumerate(group):
+                if directory.startswith("/") \
                     and exclude_external_dependencies \
-                    and dir != os.getcwd():
-                    self.printer.print_verbose(f"- Remove dir from group as it falls out of scope: {dir}")
+                    and directory != os.getcwd():
+                    self.printer.verbose(
+                        f"- Remove directory from group as it falls out of scope: {directory}"
+                        )
                     group[idx] = None
                 else:
-                    self.printer.print_verbose(f"+ Include dir: {dir}")
+                    self.printer.verbose(f"+ Include directory: {directory}")
 
             # remove the null values from the list
             group = list(filter(None, group))
@@ -97,27 +107,28 @@ class TgWrap():
         return groups
 
     def get_di_graph(self, working_dir=None):
-        """ Gets the directed graph of terragrunt dependencies, and parse it into a DG object """
-        DG = None
+        """ Gets the directed graph of terragrunt dependencies, and parse it into a graph object """
+        graph = None
         try:
             temp_file = f'{os.environ["TMPDIR"]}tg-dependencies.gv'
-            with open(temp_file, 'w') as f:
+            with open(temp_file, 'w', encoding='utf8') as f:
                 working_dir_stmt = f'--terragrunt-working-dir {working_dir}' if working_dir else ''
-                command = f'terragrunt graph-dependencies --terragrunt-non-interactive {working_dir_stmt}'
+                command = \
+                    f'terragrunt graph-dependencies --terragrunt-non-interactive {working_dir_stmt}'
                 rc = subprocess.run(
                     shlex.split(command),
                     text=True,
                     stdout=f,
                 )
-                self.printer.print_verbose(rc)
+                self.printer.verbose(rc)
 
             # Read the directed graph and reverse it
-            DG = nx.DiGraph(nx.nx_pydot.read_dot(temp_file)).reverse()
+            graph = nx.DiGraph(nx.nx_pydot.read_dot(temp_file)).reverse()
         except Exception as e:
-            self.printer.print_error(e)
+            self.printer.error(e)
             raise click.ClickException(e)
 
-        return DG
+        return graph
 
     def run_di_graph(
         self, command, exclude_external_dependencies, dry_run,
@@ -125,17 +136,19 @@ class TgWrap():
         working_dir=None,
         ):
         "Runs the desired command in the directories as defined in the directed graph"
-        DG = self.get_di_graph(working_dir=working_dir)
+        graph = self.get_di_graph(working_dir=working_dir)
 
         # first go through the groups and clean up where needed
-        groups = self.prepare_groups(graph=DG, exclude_external_dependencies=exclude_external_dependencies)
+        groups = self.prepare_groups(
+            graph=graph, exclude_external_dependencies=exclude_external_dependencies
+            )
 
         if ask_for_confirmation or self.printer.verbose:
-            self.printer.print_header(f"The following groups will be processed:")
+            self.printer.header("The following groups will be processed:")
             for idx, group in enumerate(groups):
-                self.printer.print_normal(f"\nGroup {idx}:")
-                for dir in group:
-                    self.printer.print_normal(f"- {dir}")
+                self.printer.normal(f"\nGroup {idx}:")
+                for directory in group:
+                    self.printer.normal(f"- {directory}")
 
         if ask_for_confirmation:
             response = input("\nDo you want to continue? (y/n) ")
@@ -144,46 +157,52 @@ class TgWrap():
 
         stop_processing = False
         for idx, group in enumerate(groups):
-            self.printer.print_header(f'Group {idx}')
-            self.printer.print_normal(group)
+            self.printer.header(f'Group {idx}')
+            self.printer.normal(group)
 
             if command:
-                for dir in group:
+                for directory in group:
                     # if we have a specific working dir, and the dir is relative, combine the two
-                    if working_dir and not os.path.isabs(dir):
-                        dir = os.path.join(working_dir, dir)
+                    if working_dir and not os.path.isabs(directory):
+                        directory = os.path.join(working_dir, directory)
 
-                    self.printer.print_header(f'\n\nStart processing directory: {dir}\n\n')
+                    self.printer.header(
+                        f'\n\nStart processing directory: {directory}\n\n'
+                        )
                     if dry_run:
-                        self.printer.print_warning(f'In dry run mode, no real actions are executed!!')
+                        self.printer.warning(
+                            'In dry run mode, no real actions are executed!!'
+                            )
                     else:
                         try:
                             if collect_output_file:
-                                collect_output_file.write(f'{dir}{self.SEPARATOR}')
+                                collect_output_file.write(f'{directory}{self.SEPARATOR}')
                                 collect_output_file.flush()
                             temp_file = f'{os.environ["TMPDIR"]}tgwrap-error'
                             messages = ""
-                            with open(temp_file, 'w') as f:
+                            with open(temp_file, 'w', encoding='utf8') as f:
                                 rc = {'returncode': 0}
                                 rc = subprocess.run(
                                     shlex.split(command),
                                     text=True,
-                                    cwd=dir,
-                                    stdout=collect_output_file if collect_output_file else sys.stdout, # is this really useful? Can be omitted?
+                                    cwd=directory,
+                                    stdout=collect_output_file if collect_output_file else sys.stdout,
                                     stderr=f,
                                 )
-                                self.printer.print_verbose(rc)
+                                self.printer.verbose(rc)
 
-                            with open(temp_file, 'r') as f:
+                            with open(temp_file, 'r', encoding='utf8') as f:
                                 messages = f.read()
                             if rc.returncode != 0 or 'error' in messages.lower():
-                                raise Exception("An error situation detected while processing the terragrunt graph.")
-                        except FileNotFoundError as fnfe:
-                            self.printer.print_warning(f"Directory {dir} not found, continue")
+                                raise Exception(
+                                    'An error situation detected while processing the terragrunt graph.'
+                                    )
+                        except FileNotFoundError:
+                            self.printer.warning(f'Directory {directory} not found, continue')
                         except Exception as e:
-                            self.printer.print_error(f"Error occurred:\n{str(e)}")
-                            self.printer.print_error("Full stack:")
-                            self.printer.print_normal(messages)
+                            self.printer.error(f"Error occurred:\n{str(e)}")
+                            self.printer.error("Full stack:")
+                            self.printer.normal(messages)
 
                             stop_processing = True
                             break
@@ -191,8 +210,9 @@ class TgWrap():
             if stop_processing:
                 break
 
-    def construct_command(self, command, debug, exclude_external_dependencies, non_interactive=True,
-        no_auto_approve=True, no_lock=True, update_source=False, working_dir=None, terragrunt_args=()):
+    def construct_command(self, command, debug, exclude_external_dependencies,
+        non_interactive=True, no_auto_approve=True, no_lock=True, update_source=False,
+        working_dir=None, terragrunt_args=()):
         """ Constructs the command """
         commands = {
             'generic': '{base_command} {command} --terragrunt-non-interactive {no_auto_approve} {ignore_deps} {debug_level} {update_source} {working_dir} {terragrunt_args}',
@@ -232,22 +252,25 @@ class TgWrap():
         # remove double spaces
         full_command = re.sub(' +', ' ', full_command)
 
-        self.printer.print_verbose(f"Full command to execute:\n$ {full_command}")
+        self.printer.verbose(f'Full command to execute:\n$ {full_command}')
 
         return full_command
 
-    def run(self, command, debug, dry_run, no_lock, update_source, auto_approve, working_dir, terragrunt_args):
+    def run(self, command, debug, dry_run, no_lock, update_source,
+        auto_approve, working_dir, terragrunt_args):
         """ Executes a terragrunt command on a single project """
 
-        self.printer.print_verbose(f"Attempting to 'run {command}'")
+        self.printer.verbose(f"Attempting to 'run {command}'")
         if terragrunt_args:
-            self.printer.print_verbose(f"- with additional parameters: {' '.join(terragrunt_args)}")
+            self.printer.verbose(f"- with additional parameters: {' '.join(terragrunt_args)}")
 
         check_for_file="terragrunt.hcl"
         if working_dir:
             check_for_file = os.path.join(working_dir, check_for_file)
         if not os.path.isfile(check_for_file):
-            self.printer.print_error(f"{check_for_file} is not found, this seems not to be a terragrunt project directory! Exit.")
+            self.printer.error(
+                f"{check_for_file} not found, this seems not to be a terragrunt project directory!"
+                )
             sys.exit(1)
 
         cmd = self.construct_command(
@@ -256,24 +279,24 @@ class TgWrap():
             exclude_external_dependencies=True,
             no_lock=no_lock,
             update_source=update_source,
-            no_auto_approve=False if auto_approve else True,
+            no_auto_approve=(not auto_approve),
             working_dir=working_dir,
             terragrunt_args=terragrunt_args,
         )
 
         if dry_run:
-            self.printer.print_warning(f'In dry run mode, no real actions are executed!!')
+            self.printer.warning(f'In dry run mode, no real actions are executed!!')
         else:
             rc = subprocess.run(shlex.split(cmd))
-            self.printer.print_verbose(rc)
+            self.printer.verbose(rc)
 
-    def run_all(self, command, debug, dry_run, no_lock, update_source, 
+    def run_all(self, command, debug, dry_run, no_lock, update_source,
         exclude_external_dependencies, step_by_step, working_dir, terragrunt_args):
         """ Executes a terragrunt command across multiple projects """
 
-        self.printer.print_verbose(f"Attempting to 'run-all {command}'")
+        self.printer.verbose(f"Attempting to 'run-all {command}'")
         if terragrunt_args:
-            self.printer.print_verbose(f"- with additional parameters: {' '.join(terragrunt_args)}")
+            self.printer.verbose(f"- with additional parameters: {' '.join(terragrunt_args)}")
 
         cmd = self.construct_command(
             command=command,
@@ -288,7 +311,9 @@ class TgWrap():
         )
 
         if step_by_step:
-            self.printer.print_verbose("This command will be executed in each individual project directory!")
+            self.printer.verbose(
+                'This command will be executed in each individual project directory!'
+                )
             self.run_di_graph(
                 command=cmd,
                 exclude_external_dependencies=exclude_external_dependencies,
@@ -297,17 +322,17 @@ class TgWrap():
             )
         else:
             if dry_run:
-                self.printer.print_warning(f'In dry run mode, no real actions are executed!!')
+                self.printer.warning('In dry run mode, no real actions are executed!!')
             else:
                 rc = subprocess.run(shlex.split(cmd))
-                self.printer.print_verbose(rc)
+                self.printer.verbose(rc)
 
     def analyze(self, dry_run, exclude_external_dependencies, working_dir, terragrunt_args):
         """ Analyzes the plan files """
 
-        self.printer.print_verbose(f"Attempting to 'analyze'")
+        self.printer.verbose("Attempting to 'analyze'")
         if terragrunt_args:
-            self.printer.print_verbose(f"- with additional parameters: {' '.join(terragrunt_args)}")
+            self.printer.verbose(f"- with additional parameters: {' '.join(terragrunt_args)}")
 
         # first run a 'show' and write output to file
         cmd = self.construct_command(
@@ -320,7 +345,7 @@ class TgWrap():
         # then run it and capture the output
         temp_file = f'{os.environ["TMPDIR"]}tgwrap-show'
         json_file = f'{os.environ["TMPDIR"]}planfile.json'
-        with open(temp_file, 'w') as f:
+        with open(temp_file, 'w', encoding='utf8') as f:
             self.run_di_graph(
                 command=cmd,
                 exclude_external_dependencies=exclude_external_dependencies,
@@ -336,13 +361,13 @@ class TgWrap():
                 project = split_line[0]
                 plan_file = split_line[1]
 
-                self.printer.print_header(f"Analyse project: {project}")
+                self.printer.header(f"Analyse project: {project}")
 
                 try:
                     d = json.loads(plan_file)
 
                     if 'resource_changes' in d and len(d['resource_changes']) > 0:
-                        self.printer.print_header('Changes:')
+                        self.printer.header('Changes:')
 
                         changes = False
                         for rc in d['resource_changes']:
@@ -351,22 +376,26 @@ class TgWrap():
                             if len(actions) == 1 and actions[0] == 'no-op':
                                 pass # ignore, just an state change
                             else:
-                                self.printer.print_normal(f'- {rc["address"]}: {",".join(actions)}')
+                                self.printer.normal(f'- {rc["address"]}: {",".join(actions)}')
                                 changes = True
 
                         if not changes:
                             print('- no real changes detected.')
 
                     # if so, write to file
-                    with open(json_file, 'w') as f:
+                    with open(json_file, 'w', encoding='utf8') as f:
                         f.write(plan_file)
 
                     # Check if the "TERRASAFE_CONFIG" environment variable is set
                     env_var = "TERRASAFE_CONFIG"
                     if not env_var in os.environ:
-                        self.printer.print_warning(f"{env_var} environment variable is not set, this is required for running the terrasafe command!")
+                        self.printer.warning(
+                            f"{env_var} environment variable is not set, this is required for running the terrasafe command!"
+                            )
                     else:
-                        self.printer.print_header(f"\nRun terrasafe using config {os.environ.get(env_var)}")
+                        self.printer.header(
+                            f"\nRun terrasafe using config {os.environ.get(env_var)}"
+                            )
 
                         cmd = f"cat {json_file} | terrasafe --config {os.environ.get('TERRASAFE_CONFIG')}"
                         output = subprocess.run(
@@ -377,12 +406,14 @@ class TgWrap():
                             )
                         if output.returncode != 0:
                             ts_validation_successful = False
-                            self.printer.print_error(output.stdout)
+                            self.printer.error(output.stdout)
                         elif '0 unauthorized' in output.stdout:
-                            self.printer.print_success(output.stdout)
+                            self.printer.success(output.stdout)
 
                 except json.decoder.JSONDecodeError as e:
-                    raise Exception(f"Planfile for {project} was no proper json, further analysis not possible.")
+                    raise Exception(
+                        f"Planfile for {project} was no proper json, further analysis not possible."
+                        ) from e
 
         if not ts_validation_successful:
             raise Exception("Terrasafe validation failed on one or more projects")
